@@ -26,7 +26,7 @@ def get_obo() -> Obo:
 
 def get_terms() -> Iterable[Term]:
     """Get CGNC terms."""
-    path = ensure_path(PREFIX, URL, path=f'{PREFIX}.tsv')
+    path = ensure_path(PREFIX, url=URL, path=f'{PREFIX}.tsv')
     df = pd.read_csv(path, sep='\t', dtype={'Entrez Gene id': str, 'CGNC id': str})
 
     for cgnc_id, entrez_id, ensembl_id, symbol, name, synonyms, _, _ in df.values:
@@ -45,10 +45,14 @@ def get_terms() -> Iterable[Term]:
             synonyms = []
 
         term = Term(
-            reference=Reference(prefix=PREFIX, identifier=cgnc_id, name=symbol),
+            reference=Reference(
+                prefix=PREFIX,
+                identifier=cgnc_id,
+                name=symbol if pd.notna(symbol) else None,
+            ),
             xrefs=xrefs,
             synonyms=synonyms,
-            definition=name,
+            definition=name if pd.notna(name) else None,
         )
         term.set_species(identifier='9031', name='Gallus gallus')
         yield term
