@@ -9,7 +9,10 @@ from typing import Iterable
 from tqdm import tqdm
 
 from ..path_utils import ensure_path
-from ..struct import Obo, Reference, Synonym, SynonymTypeDef, Term, from_species
+from ..struct import (
+    Obo, Reference, Synonym, SynonymTypeDef, Term, from_species, gene_product_is_a, has_gene_product,
+    transcribes_to,
+)
 
 logger = logging.getLogger(__name__)
 
@@ -38,11 +41,11 @@ gene_xrefs = [
     ('ccds', 'ccds_id'),
     ('rgd', 'rgd_id'),
     ('omim', 'omim_id'),
-    ('uniprot', 'uniprot_ids'),
-    ('ec-code', 'enzyme_id'),
-    ('rnacentral', 'rna_central_id'),
-    ('mirbase', 'mirbase'),
-    ('snornabase', 'snornabase'),
+    # ('uniprot', 'uniprot_ids'),
+    # ('ec-code', 'enzyme_id'),
+    # ('rnacentral', 'rna_central_id'),
+    # ('mirbase', 'mirbase'),
+    # ('snornabase', 'snornabase'),
 ]
 
 #: Encodings from https://www.genenames.org/cgi-bin/statistics
@@ -89,7 +92,7 @@ def get_obo() -> Obo:
         ontology=PREFIX,
         name='HGNC',
         iter_terms=get_terms,
-        typedefs=[from_species],
+        typedefs=[from_species, has_gene_product, gene_product_is_a, transcribes_to],
         synonym_typedefs=[previous_name_type, previous_symbol_type, alias_name_type, alias_symbol_type],
         auto_generated_by=f'bio2obo:{PREFIX}',
     )
@@ -98,13 +101,27 @@ def get_obo() -> Obo:
 def get_terms() -> Iterable[Term]:
     """Get HGNC terms."""
     unhandled = set()
-    path = ensure_path(PREFIX, DEFINITIONS_URL)
+    path = ensure_path(PREFIX, url=DEFINITIONS_URL)
     with open(path) as file:
         entries = json.load(file)['response']['docs']
 
     for entry in tqdm(entries, desc=f'Mapping {PREFIX}'):
         name, symbol, identifier = entry.pop('name'), entry.pop('symbol'), entry.pop('hgnc_id')[len('HGNC:'):]
 
+        relations = []
+        for uniprot_id in entry.pop('uniprot_ids', []):
+            relations.append((has_gene_product, Reference(prefix='uniprot', identifier=uniprot_id)))
+        for ec_code in entry.pop('enzyme_id', []):
+            relations.append((gene_product_is_a, Reference(prefix='ec-code', identifier=ec_code)))
+        for rna_central_id in entry.pop('rna_central_id', []):
+            relations.append((transcribes_to, Reference(prefix='rnacentral', identifier=rna_central_id)))
+        mirbase_id = entry.pop('mirbase', None)
+        if mirbase_id:
+            relations.append((transcribes_to, Reference(prefix='mirbase', identifier=mirbase_id)))
+        snornabase_id = entry.pop('snornabase', None)
+        if snornabase_id:
+            relations.append((transcribes_to, Reference(prefix='snornabase', identifier=snornabase_id)))
+
         xrefs = []
         for xref_prefix, key in gene_xrefs:
             xref_identifiers = entry.pop(key, None)
@@ -150,6 +167,9 @@ def get_terms() -> Iterable[Term]:
             synonyms=synonyms,
         )
 
+        for typedef, reference in relations:
+            term.append_relationship(typedef, reference)
+
         for prop in ['locus_group', 'locus_type', 'location']:
             value = entry.get(prop)
             if value:
